@@ -18,6 +18,7 @@ extern void F77_NAME(grpnet_binomial)(void *, void *, void *, void *, void *, vo
 extern void F77_NAME(grpnet_gamma)(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 extern void F77_NAME(grpnet_gaussian)(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 extern void F77_NAME(grpnet_invgaus)(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
+extern void F77_NAME(grpnet_multigaus)(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 extern void F77_NAME(grpnet_multinom)(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 extern void F77_NAME(grpnet_poisson)(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 extern void F77_NAME(grpnet_negbin)(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
@@ -35,6 +36,7 @@ static const R_FortranMethodDef FortranEntries[] = {
     {"grpnet_gamma",    (DL_FUNC) &F77_NAME(grpnet_gamma),    27},
     {"grpnet_gaussian", (DL_FUNC) &F77_NAME(grpnet_gaussian), 27},
     {"grpnet_invgaus", (DL_FUNC) &F77_NAME(grpnet_invgaus), 27},
+    {"grpnet_multigaus", (DL_FUNC) &F77_NAME(grpnet_multinom), 28},
     {"grpnet_multinom", (DL_FUNC) &F77_NAME(grpnet_multinom), 28},
     {"grpnet_poisson",  (DL_FUNC) &F77_NAME(grpnet_poisson),  27},
     {"grpnet_negbin",  (DL_FUNC) &F77_NAME(grpnet_negbin),  28},
